//~ #define TEST 
#define VERSION 3.5

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#define uint 	unsigned int
#define true 	1
#define false 	0
#define bool 	int

// memory management checks, in test mode only
#ifdef TEST
	uint nbMallocs=0;
	uint nbFree=0;
	void* mymalloc(size_t size){ 
		nbMallocs++; 
		return malloc(size); 
	}
	void* myrealloc(void* ptr, size_t size){ 
		if (!ptr) nbMallocs++; 
		return realloc(ptr, size); 
	}
	void myfree(void* ptr){ 
		if (ptr) {
			nbFree++; 
			free(ptr); 
		}
	}
	void printMallocStats(){
		if (nbMallocs != nbFree)
			printf("[memory] %d mallocs, %d free\n", nbMallocs, nbFree);
	}
	#define malloc(size) 		mymalloc(size)
	#define realloc(ptr, size) 	myrealloc(ptr, size)
	#define free(ptr) 			myfree(ptr)
#endif

// Class simulation
#define class(name) typedef struct name name; struct name
#define kids(T, name) T* name; uint name##Len

#define new(T) 				((T*) memset(malloc(sizeof(T)), 0, sizeof(T)))
#define clone(t) 			memcpy(malloc(sizeof(t)), &t, sizeof(t))
#define ragel(pval,peval) 	char *p = pval, *pe = peval, *eof = pe; int cs; int top; int stack[512]

// Classes
class (Slice){ 
	char 		*start, *end; 
};	// Slice methods
	#define args(sl) len(sl), (sl)->start
	int len(const Slice* sl){ 
		return sl->end - sl->start; 
	}
	bool equal(const Slice* sl, const Slice* sl2){ 
		int l = len(sl);
		return l == len(sl2) && memcmp(sl->start, sl2->start, l) == 0;
	}
class (File){
	char 		path[1024]; 
	char* 		basename; 
	int 		line; 
	Slice 		data; 
};
class (Location){
	const File* file; 
	uint 		line; 
};
class (Type){
	Slice 		name; 
	char 		isConst, pointers, arrays; 
	Slice 		arraysSpec; 
};
class (TypeRef){ 
	Slice 	name; 
	int 	pointers; 
};
class (Named){
	Slice 		name; 
};
class (Located){
	Slice 		name; 
	Location 	location;
};
class (Field){ 
	Slice 		name; 
	Location 	location; 
	Type 		type; 
	Slice 		init; 
	char 		isStatic;
};
class (Variable){
	Slice 		name; 
	Location	location; 
	Type 		type; 
};
class (Method){
	Slice 			name; 
	Location 		location; 
	Type 			returnType; 
	struct Class* 	class; 
	char 			isStatic, isInline, isConst; 
	Slice 			body; 
	uint 			bodyLine; 
	kids(Variable*, args); 
	bool 			varArgs;
	kids(Variable*, vars);
};
class (Class) { 
	Slice 			name; 
	Location 		location; 
	Slice 			parentName; 
	Class* 			parent; 
	bool			isStatic;
	kids(Method*, 	methods); 
	kids(Field*, 	fields); 
};
class (Enum) { 
	Slice 			name;
	Location 		location;
	kids(Slice*, 	constants);
};
class (Template){
	Slice 			name; 
	Location 		location; 
	Slice 			body; 
	kids(Slice*, 	args); 
};
class (TemplateInstance){ 
	Slice 			name; 
	Location		location; 
	Template* 		template; 
	Class* 			container; 
	kids(Slice*, 	args); 
};
	
// Array implementation
#define ARRAY_INCREMENT 32
#define array(T, name) T* name = 0; uint name##Len = 0
#define append(array, item) ({ if (array##Len % ARRAY_INCREMENT == 0) \
	array = realloc(array, sizeof(void*) * (array##Len + ARRAY_INCREMENT)); \
	array[array##Len++] = item; \
})
#define sort(array) qsort(array, array##Len, sizeof(array), & cmp)
#define foreach(cpt, t, array, ...) { \
	uint cpt; for(cpt=0; cpt<array##Len; cpt++){ \
	typeof(*array) t = array[cpt]; __VA_ARGS__; } \
}
#define prepend(array, item) ({ if (array##Len % ARRAY_INCREMENT == 0) \
	array = realloc(array, sizeof(void*) * (array##Len + ARRAY_INCREMENT)); \
	int i; for(i=array##Len; i>0; i--) array[i] = array[i-1]; \
	array##Len++; \
	array[0] = item; \
})
#define search(array, key) (typeof(*array))({ \
	typeof(*array) pobj = 0; \
	foreach(i, named_item, array, if (equal(&(named_item->name), key)){ pobj = named_item; break; } ) \
	pobj; \
})

// Global data
bool printLinesAndPaths = true;
struct { char *start, *end, *limit; } output = {0, 0, 0};
array(Class*, 				classes); 					
array(Enum*, 				enums); 					
array(Template*, 			templates);				
array(TemplateInstance*, 	templateInstances);
array(Slice*, 				macros);
array(File*, 				files);		
array(File*, 				tplInstFiles);		
array(Class*, 				tplInstClasses); 					

char _INT[] 	= "int"; 		Slice INT 		= {_INT, _INT+3};
char _DOUBLE[] 	= "double"; 	Slice DOUBLE 	= {_DOUBLE, _DOUBLE+6};
char _CHAR[] 	= "char"; 		Slice CHAR 		= {_CHAR, _CHAR+4};
char _BOOL[] 	= "bool"; 		Slice BOOL 		= {_BOOL, _BOOL+4};

int min(int a, int b){ return a<b? a: b; }
#define die(...) printf("[gt] " __VA_ARGS__), puts(""), fflush(stdout), exit(1)
void panic(const Location* loc, const char* p, const char* pe, const char* format, ...){
	// Calculate column number
	const char* start = loc->file->data.start;
	const char* ptr = p;
	while(*ptr != '\n' && ptr > start) ptr--;
	int col = p - ptr;

	// Produce the error message
	printf("%s:%d:%d: ", loc->file->path, loc->line, col);
	va_list args; va_start(args, format); vprintf(format, args); va_end(args);
	puts("");
	if (p != pe){
		const char* end; for(end = p; end < pe; end++){ // find EOL / EOF before pe
			switch(*end){
				case '\n': case '\r': pe = end; break;
				case 0: case -1: end = pe; break;
			}
		}
		printf("    at [%.*s]\n", (int)(pe-p), p);
	}
	fflush(stdout);
	exit(1);
}

// Output methods
void grow(){
	if (output.limit - output.end < 8192){
		int len = output.end - output.start;
		int capa = output.limit - output.start;
		capa = capa * 2 + 1000000;
		output.start = realloc(output.start, capa);
		if (!output.start) die("couldn't realloc %d bytes", capa);
		output.end = output.start + len;
		output.limit = output.start + capa;
	}
}
void out(const Slice* s){
	grow();
	int slen = len(s);
	output.end = (char*) memcpy(output.end, s->start, slen) + slen;
}
void outs(const char* s){
	grow();
	int slen = strlen(s);
	output.end = (char*) memcpy(output.end, s, slen) + slen;
}
void outc(char c){
	grow();
	*output.end++ = c;
}
#define outf(...) grow(), output.end += sprintf(output.end, __VA_ARGS__)
void outt(const Type* t){
	if (t->isConst) outs("const ");
	out(&t->name);
	int ptrs = t->pointers + t->arrays;
	while(ptrs--) outc('*');
}
void outt_noarrays(const Type* t){
	if (t->isConst) outs("const ");
	out(&t->name);
	int ptrs = t->pointers;
	while(ptrs--) outc('*');
}
void outm(const Method* m){
	outf("%.*s_%.*s", args(&m->class->name), args(&m->name));
}

// Global functions
char* replace(char* str, char from, char to){ // replace 'from' by 'to' in 'str' (Cstring) 
	char* p; for(p = str; *p; p++) if (*p == from) *p = to;
	return str;
}
void firstPass(const File* file);
bool addpath(const char* path){ // loads file contents into ram for further processing
	File* file = new(File);
	// Path conversion (\ to / if any)
	char* cpathEnd = file->path;
	file->basename = cpathEnd;
	int c; while ((c = *path++)){
		if (c == '/' || c == '\\'){ 
			*cpathEnd++ = '/'; 
			while (*path == '\\') path++; 
			file->basename = cpathEnd; 
		}
		else *cpathEnd++ = c;
	}
	*cpathEnd = 0;
	
	// Load each file only once
	foreach(i, f, files, if (strcmp(f->basename, file->basename) == 0) { free(file); return true; })
		
	// File open & size
	FILE* f = fopen(file->path, "r");
	if (! f) return false;
	fseek(f, 0, SEEK_END);
	int flen = ftell(f)+512; // just in case
	fseek(f, 0, SEEK_SET);
	// File copy to ram
	file->line = 1;
	file->data.start = (char*) malloc(flen);
	file->data.end = file->data.start + fread(file->data.start, sizeof(char), flen, f);
	if (!feof(f)) die("problem reading %s", file->path);
	fclose(f);
	firstPass(file);
	append(files, file);
	return true;
}

%%{ machine common;
	action s{ s.start = p; }
	action se{ s.end = p; }
	action echo{ ECHO; }
	action fhold{ fhold; }
	
	newline = '\n' @{ loc.line++; };
	anynl = any | newline;		
	
	blockCommentBody := anynl* $echo :>> '*/' $echo @{ fret; };
	blockComment = '/*' $echo @{ fcall blockCommentBody; }; 

	lineCommentBody := any* $echo :>> newline $echo @{ fret; };
	lineComment = '//' $echo @{ fcall lineCommentBody; }; 

	s = (space | newline | lineComment | blockComment) @echo;

	bool = ('true' | 'false') $echo;
	char = ('\'' '\\'? [^\n] '\'') $echo;
	int = ('-'? digit+) $echo;
	decimal = ('.' digit+) $echo; # follows an int to represent a double (no race)
	
	string = '"' $echo @{ fcall stringBody; };
	stringBody := (anynl>1 | ('\\' [^\n]) >2)* $echo :>> '"' $echo @{ fret; };

	identifier = (alpha | '_') (alnum | '_')*;
	word = identifier >s %se;

	keywords = 'return' | 'if' | 'else' | 'do' | 'while' | 'for' | 'switch' 
		| 'case' | 'break' | 'default' | 'static' | 'const' | 'inline' | 'using';

	type = (
		('const' s+)? @{ t.isConst = true; } 
		(identifier - keywords) >{t.name.start = p;} %{t.name.end = p;} 
		(s | '*' @{ t.pointers++; })*  
	)>{ memset(&t, 0, sizeof(Type)); };

	arraysSpec = (s*'['s* (digit* | identifier) s*']' >{ t.arrays++; })*
		>{t.arraysSpec.start = p;} 
		%{t.arraysSpec.end = p;} ;
		
	macro = '#' >s $echo @{ fcall macroE; } %se %{ append(macros, clone(s)); };
	macroLFE := '\n' @{ fgoto macroE; };
	macroE := any* $echo %eof{ fhold; fret; } :>>
		(
			('\\\r') $echo @{ fgoto macroLFE; } # handle unix/windows end of line in macros
			| ('\\' anynl) $echo @{ fgoto macroE; }
			| newline @{ loc.line--; fhold; fret;}
		);		
}%%

void instantiate(TemplateInstance* t){
	// Allocate enough to expand variables of the template
	Template* tpl = t->template;
	File* f = append(tplInstFiles, new(File));
	strcpy(f->path, tpl->location.file->path);
	f->line = tpl->location.line;
	f->data.end = f->data.start = malloc(sizeof(char) * (len(&tpl->body) * 3 + 100));
	f->data.end += sprintf(f->data.end, "class %.*s", args(&t->name));
	
	// Template engine: copy template chars but expand ${} expressions
	// ${paramName} or ${templateName} are expanded
	Location loc = tpl->location;
	ragel(tpl->body.start, tpl->body.end);
	Slice s = {0,0};
	Slice type_name = {0, 0};
	bool escape = false; // escape * into _ during name expansion
	#define ECHO *f->data.end++ = *p
	%%{ machine gtInstantiateTemplate;
		include common;
		action esc{ escape = true; }
		action ts{ type_name.start = p; }
		action tse{ type_name.end = p; }
		action eval{
			f->data.end--; // remove '$'
			
			if (equal(&s, &tpl->name)){ // ${templateName}
				int l = len(&t->name);
				f->data.end = (char*) memcpy(f->data.end, t->name.start, l) + l;
			} else { // ${paramName}
				int index = -1;
				foreach(i, arg, tpl->args, if (equal(&s, arg)) { index = i; break; } )
				if (index == -1)
					panic(&t->location, p, pe, "unknown template name %.*s", args(&s));

				Slice* val = t->args[index];
				if (type_name.start){ // replace ${T=xxx} by 1 or 0
					*f->data.end++ = 
						(type_name.start[0] == '*')? // replace ${PointerType=*} by 1, ${PointerType=*} by 0
							val->end[-1] == '*'? '1': '0':
						equal(val, &type_name)? // replace ${T=SomeType} by 1 or 0
							'1': '0'; 
				} else if (escape){ // replace ${$T} by ValueType or PointerType_
					char* ptr = val->start; 
					char* end = val->end;
					while (ptr<end){ // replace ${T} by ValueType
						*f->data.end++ = *ptr == '*'? '_': *ptr;
						ptr++;
					}
				} else {
					int l = len(val);
					f->data.end = (char*) memcpy(f->data.end, val->start, l) + l;
				}
			}
			type_name.start = type_name.end = 0;
			escape = false;
		}
		main := (anynl* $echo :>> 
			'${' ('$'>esc)? identifier>s%se ('=' (identifier? '*'?)>ts%tse)? '}'%eval)* 
			(anynl* -- '${') $echo;
		write data nofinal; write init; write exec;
	}%%
	#undef ECHO	
	if (cs < %%{ write first_final; }%%) 
		panic(&loc, p, pe, "template instantiation error");
	
	*f->data.end++ = '}';
}

void firstPass(const File* file){ // parse only templates and includes
	Location loc = {file, file->line};
	ragel(file->data.start, file->data.end);
	Slice s; 
	Template* tpl=0;
	
	#define ECHO
	%%{ machine gtTemplatesIncludes;
		include common;
		
		imports = 'import' s+ string >s %se %{ 
			char* endDir = strrchr(loc.file->path, '/');
			char newPath[1024];
			sprintf(newPath, "%.*s%.*s", (int)(endDir?endDir+1-loc.file->path: 0), loc.file->path, // dir/ or nothing
				(int)(s.end-s.start-2), s.start+1); // remove " " 
			if (! addpath(newPath))
				panic(&loc, p, pe, "file not found %s", newPath);
		};
		
		tplArg = s* word %{ append(tpl->args, clone(s)); };
		template = 'template' s* word %{
			tpl = append(templates, new(Template));
			tpl->location = loc; 
			tpl->name = s; 
		} s* '<' tplArg (s* ',' tplArg)* s* '>' @{ fgoto templateE; };
		templateE := anynl* >s :>> '}<>' >se @{ tpl->body = s; fgoto main; };
		
		main := ( s%2 | string$2 | char$3 | template%1 | imports%1 | anynl%1 )*;
		write data nofinal; write init; write exec;
	}%% 
	#undef ECHO
	if (cs < %%{ write first_final; }%%) 
		panic(&loc, p, pe, "template or include error");
}

void secondPass(const File* file, bool isTemplate){ // parse everything but method bodies
	Location loc = {file, file->line};
	ragel(file->data.start, file->data.end);
		
	Class* c=0; Method* m=0; Field* f=0; Slice s; Type t; 
	Enum* e=0;
	TemplateInstance* tplInst=0;
	char isStatic = false, isInline = false;
	int nbBlocks = 0;

	#define ECHO
	%%{ machine gtStructure;
		include common;
		
		usingArg = (any - s)+ >s %se %{ if (tplInst) append(tplInst->args, clone(s)); } 
			:>> (','|'>'|space) @fhold;
		usingE := s* word %{ // Template instanciation
			tplInst = append(templateInstances, new(TemplateInstance)); 
			tplInst->location = loc;
			tplInst->name = s;
			tplInst->container = c;
			foreach(i, t, templateInstances, // remove duplicates
				if (i == templateInstancesLen-1) break; // don't compare to itself
				if (equal(&t->name, &s)) { free(tplInst); tplInst = 0; templateInstancesLen--; break; } 
			)
		} s* '=' s* word %{
			if (tplInst){
				if (! (tplInst->template = search(templates, &s)))  // find template
					panic(&loc, p, pe, "unknown template %.*s", args(&s));
			}
		} s* '<' s* usingArg (s* ',' s* usingArg)* s* '>' @{
			if (tplInst){
				Slice s = tplInst->template->name;
				if (tplInst->template->argsLen != tplInst->argsLen) 
					panic(&loc, p, pe, "bad number of template arguments");
				instantiate(tplInst);
			}
			fret;
		};
		using = 'using' %{ fcall usingE; };
		
		static = ('static' s+) @{ isStatic = true; };
		inline = ('inline' s+) @{ isInline = true; };
		const = ('const') @{ m->isConst = true; };
		
		field = static? type s+ word s* arraysSpec s* %{
			if (!isStatic && c->isStatic) 
				panic(&loc, p, pe, "static classes can only contain static fields");
			f = append(c->fields, new(Field));
			f->location = loc;
			f->type = t;
			f->name = s;
			f->isStatic = isStatic;
		} (
			'=' anynl+ >{ // cannot use complex expressions here because 1st pass not finished
				if (! isStatic) panic(&loc, p, pe, "can only initialize static fields");
				f->init.start = p;
			} %{f->init.end = p;} 
		)? :>> ';' @{isStatic = false;};
		
		arg = type s+ word %{
			Variable* arg = append(m->args, new(Variable));
			arg->location = loc;
			arg->type = t;
			arg->name = s;
		} s*;
		methodSignature = static? inline? type s+ word s* const? s* '(' %{
			if (search(c->methods, &s)) panic(&loc, p, pe, "method already defined");
			if (!isStatic && c->isStatic) 
				panic(&loc, p, pe, "static classes can only contain static methods");
			m = append(c->methods, new(Method));
			m->location = loc;
			m->isStatic = isStatic;
			m->isInline = isInline;
			m->returnType = t;
			m->name = s;
			m->class = c;
			isStatic = isInline = false;
		} s* (arg (',' s* arg)*)? (',' s* '...' @{ m->varArgs = true; })? ')' s* const?;
		methodBodyE := ('{' @{nbBlocks++;} | s | string$2 | char$3 | '}' @{if (! --nbBlocks) fret;} | any)*;
		methodBody = '{' @{ nbBlocks++; fcall methodBodyE; } >{ 
			Slice* s = &m->body;
			m->bodyLine = loc.line;
			s->start = p;
		} %{ 
			m->body.end = p;
		};
		method = methodSignature s* methodBody;

		classSignature = s+ word %{
			c = search(classes, &s);
			if (! c){
				if (isTemplate) c = append(tplInstClasses, new(Class));
				else c = append(classes, new(Class));
				c->location = loc;
				c->name = s; 
				c->isStatic = isStatic;
			}
		} s* (':' s* word %{ c->parentName = s; } )?;
		class = ('static' @{isStatic = true;} s*)? 
			'class' classSignature '{' @{ isStatic = false; } s* 
				(field | method | using | s)* 
			'}' @{ c = 0; };
			
		# skip imports and templates (already parsed in first pass)
		imports = 'import' any* :>> newline;
		template = 'template' anynl* :>> '}<>';

		enum = 'enum' s+ word %{
				e = append(enums, new(Enum));
				e->location = loc;
				e->name = s;
			}'{' s* word %{ append(e->constants, clone(s)); } s* 
			(',' s* word %{ append(e->constants, clone(s)); } s*)* '}';
			
		main := ( class | using | macro | imports | template | enum | s)*;

		write data nofinal; write init; write exec;
	}%% 
	#undef ECHO
	if (cs < %%{ write first_final; }%%) 
		panic(&loc, p, pe, "structure error");
}
Method* getMethod(const Slice* className, const Slice* methodName){
	Class* p = search(classes, className);
	while(p){ // search in inheritance tree
		Method* m = search(p->methods, methodName);
		if (m) return m;
		else p = p->parent;
	}
	return 0;
}
Field* getField(const Slice* className, const Slice* fieldName){
	Class* p = search(classes, className);
	while(p){ // search in inheritance tree
		Field* f = search(p->fields, fieldName);
		if (f) return f;
		else p = p->parent;
	}
	return 0;
}
int nbEmptyExpressions = 0;
char* parseExpression(const Method* m, char* pCtx, uint* pline, TypeRef* pt, char* sub){
	// sub = start of the previous "sub expression". Location where to insert instance method calls
	#define setPt(typeName, nbPointers) pt->name = typeName, pt->pointers = nbPointers
	
	Location loc = {m->location.file, *pline};
	ragel(pCtx, m->body.end);
	
	static char exprData[8192];
	static Slice expr = {exprData,exprData};
	bool isRoot = pt == 0;
	if (isRoot) expr.end = expr.start;
	
	Slice s = {0,0};
	char isConst = false;
	
	TypeRef exprT, subT;
	memset(&exprT, 0, sizeof(TypeRef));
	memset(&subT, 0, sizeof(TypeRef));
	if (isRoot) pt = &exprT;
	Type t;
	Field* field = 0;
	Method* method = 0;
	
	#define ECHO *expr.end++ = *p
	char* argsStart = 0;
	int nbBlocks = 0;
	%%{ machine gtExpression; 
		include common;
		
		action expression{
			fexec parseExpression(m, p+1, &loc.line, &subT, 0); 
		}
		action expressionExt{
			fexec parseExpression(m, p+1, &loc.line, &subT, sub); 
		}		
		op = ('+' | '-'  | '*'  | '/'  | '%'  | '<'  | '>'  
			| '==' | '=' | '&' | '|' | '[' | ']' | '!' | '~' | '^' | '?' | ':') $echo;
		args = '(' @echo s* ( ')' | (any-s-')') @fhold @expression 
			(',' @echo s* (any-s) @fhold @expression s*)* ')' ) @echo;
		
		invocationArgs = '(' s* ( ')' | 
			(any-s-')') @fhold @{ argsStart = expr.end; *expr.end++ = '('; } 
				@expression (',' @echo s* (any-s) @fhold @expression)* ')' 
		) @echo;
		
		invocation = ('.' s* word %{ // insert "method" call at start of last expression
			if (!len(&pt->name)) panic(&loc, p, pe, "unknown target type for method call [%.*s]", args(&s));
			
			field = getField(&pt->name, &s);
			if (field){ // this is a field
				if (field->isStatic) *expr.end++ = '_';
				else {
					if (pt->pointers) *expr.end++ = '-', *expr.end++ = '>';
					else *expr.end++ = '.';
				}
				int l = len(&s);
				expr.end = (char*) memcpy(expr.end, s.start, l) + l;
				setPt(field->type.name, field->type.pointers + field->type.arrays);
			}
			argsStart = 0;
		} s* (invocationArgs %{
			if (sub > expr.end) panic(&loc, p, pe, "internal error in expression");
			
			method = getMethod(&pt->name, &s);
			if (!method) panic(&loc, p, pe, "unknown member [%.*s.%.*s]", args(&pt->name), args(&s));

			Slice* mName = &method->name;
			int mNameLen = len(mName);
			int tNameLen = len(&pt->name);
			if (method->isStatic){ // static method
				char* dest = sub + tNameLen; // Type name already written
				int offset = mNameLen + 1 + (argsStart? 0: 1); // to insert mName_(?
				memmove(dest + offset, dest, expr.end - dest);
				*dest++ = '_'; // type/method separator (name mangling)
				memcpy(dest, mName->start, mNameLen);
				if (!argsStart) dest[mNameLen] = '(';
				expr.end += offset;
			} else {
				if (argsStart) *argsStart = ','; // if at least one arg replace '(' by ',' (after target object)				
				int makeptr = pt->pointers? 0: 1;
				int fullNameLen = tNameLen + 1 + mNameLen; // type_method
				int offset = fullNameLen + 1 + makeptr; // +1 for the '(', +makeptr for the '&'
				memmove(sub + offset, sub, expr.end - sub);
				char* dest = (char*) memcpy(sub, pt->name.start, tNameLen) + tNameLen;
				*dest++ = '_';
				dest = (char*) memcpy(dest, mName->start, mNameLen) + mNameLen;
				*dest++ = '(';
				if (makeptr) *dest = '&';
				expr.end += offset;
			}
			setPt(method->returnType.name, method->returnType.pointers + method->returnType.arrays);
		})? ) %{
			if (!field && !method)
				panic(&loc, p, pe, "unknown member [%.*s.%.*s]", args(&pt->name), args(&s));
			field = 0;
			method = 0;
		};
		
		action ref{ // Class, var ref or fct call
			bool found = false;
			#define ok(name, pointers) setPt(name, pointers), found = true
			
			Variable* v = search(m->vars, &t.name); // var ref
			if (!v) // or arg ref
				foreach(i, arg, m->args,
					if (equal(&arg->name, &t.name)){ v = arg; break; }
				)
			if (v) ok(v->type.name, v->type.pointers + v->type.arrays);
			else {
				Class* c = search(classes, &t.name);
				if (c) ok(c->name, -1); // Class ref
			}
			
			if (!found) setPt(t.name, t.pointers + t.arrays); // cast
		}
		compoundExpression = '{' @echo @{nbBlocks++;} @expression
			| '}' @echo @{nbBlocks--; if (nbBlocks == 0) return p; } @expression
			| ',' @{ if (nbBlocks == 0) { if (isRoot) out(&expr); return p;} else {ECHO;}} @expression;

		main := (compoundExpression |
			(( 
				'(' @echo @expression @{*pt = subT;} ')' @echo @expressionExt
				| int %{setPt(INT, 0);} decimal? %{setPt(DOUBLE, 0);}
				| string %{setPt(CHAR, 1);} 
				| bool %{setPt(BOOL, 0);} 
				| char %{setPt(CHAR, 0);} 
				| 'this' %{ setPt(m->class->name, 1); }
				| 'new' s* '(' s* type s* ')' @{ setPt(t.name, 1); }
				| 'super' %{
					Class* c = m->class->parent;
					if (! c) panic(&m->location, p, pe, 
						"cannot use 'super': %.*s has no superclass", args(&m->class->name));
					else setPt(c->name, 1);
				}
				| (type - 'this' - 'super') $echo %ref args?
			) >{ sub = expr.end; } # save current output position before expression
			| op
			| s
			| invocation
			)-- blockComment -- lineComment 					
			# remove comments because of a conflict with operator '/'
		)** [;)] @{
			*pline = loc.line;
			if (isRoot) out(&expr); 
			return p; 
		};

		write data nofinal; write init; write exec;
	}%%
	#undef ECHO
	
	if (cs < %%{ write first_final; }%% || isRoot)
		panic(&loc, p, pe, "expression error");
	return p;
}
#define parseExpression(m, p, l) parseExpression(m, p, l, 0, 0)
void parseBody(Method* m){
	Location loc = {m->location.file, m->bodyLine};
	ragel(m->body.start, m->body.end);
	
	Slice s = {0,0}; Type t;
	char isConst = false;
	
	char* start=0; int lineAtStart=0; char* outEnd=0; 
	bool isStatic;
	#define ECHO outc(*p)
	%%{ machine gtBody;
		include common;
		
		action expression{ fexec parseExpression(m, p+1, &loc.line); }
		action mark{ start = p; lineAtStart = loc.line; outEnd = output.end; }

		static = ('static' s+) @{ isStatic = true; };

		action var{
			if (s.end > s.start){
				Variable* var = append(m->vars, new(Variable)); 
				var->location = loc;
				var->name = s; 
				var->type = t;
			}
		}
		
		statement = 
			('{' $echo | '}' $echo | ';' $echo | ',' $echo 
				| 'return' $echo @expression
				| 'if' $echo s* '(' $echo @expression ')' $echo
				| 'else' $echo
				| 'do' $echo s* '{' $echo
				| 'while' $echo s* '(' $echo @expression ')' $echo
				| 'for' s* '(' @{output.end = outEnd;} 
					@expression (',' $echo @expression)* ';' $echo @{outs(" for(;");} 
					@expression ';' $echo
					@expression (',' $echo @expression)* ')' $echo
				| 'switch' $echo s* '(' $echo @expression ')' $echo s* '{' $echo
				| 'case' $echo s* (char | int | identifier $echo) s* ':' $echo 
				| 'break' $echo
				| 'default' $echo s* ':' $echo 
				| (static? type >{s.end = s.start;} s+ word? s* arraysSpec 
					('=' $echo @expression)? s* %var
					(',' s* word s* arraysSpec ('=' $echo @expression)? %var)* ';'
				) $echo $err{
					loc.line = lineAtStart; output.end = outEnd;
					p = parseExpression(m, start+1, &loc.line) - 1;
					if (p <= start) panic(&loc, p, pe, "invalid statement structure");
					fgoto main;
				}
				| ('(' | '&' | '\'' | '"' | '-' | '+' | '*' | '!' | digit) @fhold @expression
			) >to(mark); 
			
		main := (s | statement | macro)*;

		write data nofinal; write init; write exec;
	}%%
	#undef ECHO	
	if (cs < %%{ write first_final; }%%) 
		panic(&loc, p, pe, "method body error");
}

void lineAndPath(const Location* loc){
	if (printLinesAndPaths)
		outf("#line %d \"%s\"\n", loc->line, loc->file->path);
}
void printMethodSignature(const Method* m, bool indent){
	lineAndPath(&m->location);
	if (indent) outc('\t');
	if (m->isInline) outs("inline ");
	outf("%s%.*s", m->returnType.isConst? "const ": "", args(&m->returnType.name));
	{int ptrs = m->returnType.pointers + m->returnType.arrays; while(ptrs--) outc('*'); }
	outc(' ');
	outm(m);
	outc('(');
	if (! m->isStatic){
		if (m->isConst) outs("const ");
		outf("%.*s* this", args(&m->class->name));
		if (m->argsLen) outs(", ");
	}
	foreach(k, a, m->args,
		outt(&a->type);
		outf(" %.*s", args(&a->name));
		if (k < m->argsLen - 1) outs(", ");
	)
	if (m->varArgs) outs(", ...");
	outc(')');
}
void printFields(const Class* c){
	if (c->parent) printFields(c->parent);
	foreach(j, f, c->fields, if (! f->isStatic) { 
		if (printLinesAndPaths){
			outc('\t');
			lineAndPath(&f->location);
		}
		outc('\t');
		outt_noarrays(&f->type); 
		outf(" %.*s%.*s;\n", args(&f->name), args(&f->type.arraysSpec));
	} )
}
void printInheritedMethods(const Class* c, const Class* ancestor){
	if (ancestor->parent) printInheritedMethods(c, ancestor->parent);
	foreach(j, m, ancestor->methods,
		// First, undefine method if inherited
		if (ancestor->parent){
			outf("#ifdef %.*s_%.*s\n", args(&c->name), args(&m->name));
			outf("\t#undef %.*s_%.*s\n", args(&c->name), args(&m->name));
			outs("#endif\n");
		}
		// Then (re)define it
		outf("#define %.*s_%.*s(...) ", args(&c->name), args(&m->name));
		outf("%.*s_%.*s((%.*s*) __VA_ARGS__)\n", args(&m->class->name), args(&m->name), args(&ancestor->name));
	)
}
void printMethods(const Class* c){
	if (c->parent) printInheritedMethods(c, c->parent);
	foreach(j, m, c->methods,
		if (c->parent){
			outs("#ifdef "); outm(m); outs("\n");
			outs("\t#undef "); outm(m); outs("\n");
			outs("#endif\n");
		}
		printMethodSignature(m, false);
		outs(";\n");
		if (len(&m->name) == 4 && !strncmp(m->name.start, "main", 4)){
			outs("#define "), outm(m), outs(" main\n");
		}
	)
}
bool isPrimitive(const Slice* typename){
	ragel(typename->start, typename->end);
	%%{ machine primitiveType;
		main := 'bool' | 'float' | 'double' | 'void' |
			'char' | 'short' | 'int' | 'long' |
			'uchar' | 'ushort' | 'uint' | 'ulong';
		write data nofinal; write init; write exec;
	}%%
	return cs >= %%{ write first_final; }%%;
}
void thirdPass(){
	outs("/***** Macros *****/\n");
	outs("#define new(T) (T*) malloc(sizeof(T))\n");
	foreach(i, m, macros, outf("%.*s\n", args(m)))
	
	outs("\n/***** Type declarations *****/\n");
	outs("typedef unsigned char uchar;\n" "typedef unsigned short ushort;\n" 
		"typedef unsigned int uint;\n" "typedef unsigned long ulong;\n" 
		"typedef unsigned int bool;\n" "#define null 0\n"
		"#define true 1\n" "#define false 0\n");
	const char* primitives[] = { "double", "float", "short", "ushort", 
		"int", "uint", "long", "ulong", "bool", "char"};
	int primitivesLen = sizeof(primitives) / sizeof(char*);
	foreach(i, p, primitives, 
		outf("#define %s(value) (%s*)({%s v=value; (int)&v;})\n", p, p, p)
	)

	outs("\n/***** Enums *****/\n");
	foreach(i, e, enums, 
		lineAndPath(&(e->location));
		outf("typedef enum %.*s{\n\t", args(&e->name));
		foreach(i, c, e->constants, 
			outf("%.*s", args(c));
			if (i < e->constantsLen - 1) outs(", ");
		)
		outf("\n} %.*s;\n", args(&e->name));
	)
		
	//***** merge classes and templates instances classes *****
	array(Class*, allClasses);
	int i; 
	for(i=tplInstClassesLen-1; i>=0; i--) append(allClasses, tplInstClasses[i]);
	for(i=0; i<classesLen; i++) append(allClasses, classes[i]);
	#ifdef TEST 
		free(classes); // nice cleanup (optional)
		free(tplInstClasses); // nice cleanup (optional)
	#endif
	classes = allClasses;
	classesLen = allClassesLen;
	
	outs("\n/***** Type (forward) declarations *****/\n");
	foreach(i, c, classes, 
		if (!isPrimitive(&c->name) && !c->isStatic){
			lineAndPath(&c->location);
			outf("typedef struct %.*s %.*s;\n", args(&c->name), args(&c->name));
		}
	)
	
	foreach(i, c, classes, // resolve inheritance reference if any
		if (len(&c->parentName) && !c->parent)
			if (! (c->parent = search(classes, &c->parentName)))
				panic(&c->location, c->parentName.start, c->parentName.end, 
					"unknown parent type [%.*s]", args(&c->parentName));
	)
	outs("\n/***** Type definitions *****/\n");
	foreach(i, c, classes, 
		if (!isPrimitive(&c->name) && !c->isStatic){
			lineAndPath(&c->location);
			outf("struct %.*s{\n", args(&c->name));
			printFields(c);
			outs("};\n");
		}
	)
	
	outs("\n/***** Static fields *****/\n");
	foreach(i, c, classes,
		foreach(j, f, c->fields, if (f->isStatic) {
			lineAndPath(&f->location);
			outt_noarrays(&f->type);
			outf(" %.*s_%.*s%.*s", args(&c->name), args(&f->name), args(&f->type.arraysSpec));
			if (len(&f->init)) outf("=%.*s", args(&f->init));
			outs(";\n"); 
		})
	)

	outs("\n/***** Method declarations *****/\n");
	foreach(i, c, classes,
		outf("/* Class: %.*s */\n", args(&c->name));
		printMethods(c);
	)
			
	outs("\n/***** Method definitions *****/\n");
	foreach(i, c, classes,
		outf("/* Class: %.*s */\n", args(&c->name));
		if (c->methodsLen){
			if (c->parent) outf("#define super ((%.*s*) this)\n", args(&c->parent->name));
			foreach(j, m, c->methods,
				printMethodSignature(m, true);
				parseBody(m);
				outc('\n');
			)
			if (c->parent) outs("#undef super\n");
		}
	)
}
#ifndef __APPLE__
	void* __stdcall LoadLibraryA(const char* path);
	void* __stdcall GetProcAddress(void*, const char* fct); 
#endif
int main(int argc, char** argv) {
	if (argc > 1 && strcmp(argv[1], "-noline") == 0){ // remove #line pragmas
		printLinesAndPaths = false;
		argv++; argc--;
	}
	if (argc < 3) die("Usage: gt [-noline] input.g output.c | - | -run");
	
	char* path = argv[1];
	if (! addpath(path)) die("problem adding %s", path); // first pass: imports, templates
	
	int i; 
	for(i=0; i<filesLen; i++) secondPass(files[i], false); // structure
	for(i=0; i<tplInstFilesLen; i++) secondPass(tplInstFiles[i], true); // structure
	thirdPass(); // methods bodies
	
	const char* outpath = argv[2];
	if (strcmp(outpath, "-") == 0){
		fwrite(output.start, sizeof(char), output.end - output.start, stdout);
	} else if (strcmp(outpath, "-run") == 0){
		#ifndef __APPLE__
			outc(0); // end of char*
			const char* tccPath = "c:/tcc/tcc.dll";
			void* dll = LoadLibraryA(tccPath);
			if (! dll) { die("couldn't load %s to interpret program\n", tccPath); }
			void* state = ((void* (*)()) GetProcAddress(dll, "tcc_new"))(); 
			((int (*)(void*, int)) GetProcAddress(dll, "tcc_set_output_type"))(state, 0); // TCC_OUTPUT_MEMORY 
			((void (*)(void*, const char*)) GetProcAddress(dll, "tcc_set_lib_path"))(state, "c:/tcc"); 
			((int (*)(void*, const char*)) GetProcAddress(dll, "tcc_add_include_path"))(state, "."); 
			((int (*)(void*, const char*, int)) GetProcAddress(dll, "tcc_set_warning"))(state, "all", 1); 
			((int (*)(void*, const char*, int)) GetProcAddress(dll, "tcc_set_warning"))(state, "error", 1); 
			if (! ((int (*)(void*, char*)) GetProcAddress(dll, "tcc_compile_string"))(state, output.start)){
				argc -= 2;
				argv += 2;
				((int (*)(void*, int, char**)) GetProcAddress(dll, "tcc_run"))(state, argc, argv);
			}
			((void (*)(void*)) GetProcAddress(dll, "tcc_delete"))(state);
		#else
			die("tcc isn't yet integrated on MacOSX"); 
		#endif
	} else {
		FILE* fout = fopen(outpath, "w");
		if (! fout) die("couldn't open %s for writing", outpath); 
		fwrite(output.start, sizeof(char), output.end - output.start, fout);
		fclose(fout);
	}
	#ifdef TEST
	{ // nice cleanup (optional)
		#define nfree(array) foreach(i, t, array, free(t)); free(array);
		free(output.start);
		nfree(macros);
		foreach(i, e, enums, nfree(e->constants));
		nfree(enums);
		foreach(i, tpl, templates, nfree(tpl->args));
		nfree(templates);
		foreach(i, tpl, templateInstances, nfree(tpl->args));
		nfree(templateInstances);
		foreach(i, c, classes,
			nfree(c->fields);
			foreach(j, m, c->methods, nfree(m->args); nfree(m->vars))
			nfree(c->methods);
		)
		nfree(classes);
		foreach(i, f, files, free(f->data.start))
		nfree(files);
		foreach(i, f, tplInstFiles, free(f->data.start))
		nfree(tplInstFiles);
		printMallocStats();
	}	
	#endif
	return 0;
}
